// MMC.H header file 
// Initialize , Read and write MMC Card

//#include <stdio.h>
//#include <stdlib.h>
#include <delay.h> 
//#include <string.h> 

#define IDLE_STATE				0			// MMC to SPI
#define INIT_COMMAND			1			// initialize card
#define CRC_OFF					59			// CRC checking on/off (normally automatically off in SPI)
#define SET_BLKLEN				16			// Set number of bytes to transfer per block (default 512 in SPI)
#define WRITE_BLK				24			// write a block
#define READ_BLK				17			// read a block

// Data tokens values
#define STARTBYTE_RDWR		0xFE		// startbyte, block starts with 0xFE (both write and read block)
#define DR_MASK					0x1F
#define DR_ACCEPT				0x05

#ifndef BLOCKSIZE
	#define BLOCKSIZE			512          
#endif

#ifndef CS
	#define CS PORTB.0
#endif
	
// *********** PROTOTYPES ***********
 // Write command to MMC, returns R1 result code, leaves ENABLE ON
unsigned char mmc_command(unsigned char cmd, unsigned long arg);  
// Initialize mmc-card, returns zero if successful.
unsigned char mmc_init(void);
// Read buffer, returns zero if successfull
unsigned char mmc_read(unsigned long sector, unsigned char* buffer); 
// write to MMC from buffer
unsigned char mmc_write(unsigned long sector, unsigned char* buffer);

// *********** DEFINITIONS ***********               


// Write command to MMC, returns R1 result code, leaves ENABLE ON
unsigned char mmc_command(unsigned char cmd, unsigned long arg)
{
	unsigned char resp;
	unsigned char retry=0;     
	
	CS = 0;
	spi(cmd | 0x40);                                // note command is CMD-X + 0x40
	spi(arg>>24);
	spi(arg>>16);
	spi(arg>>8);
	spi(arg);
	spi(0x95);                                      // spi CRC = always 0xFF OR 0x95 for MMC_GO_IDLE_STATE command
	while((resp = spi(0xFF)) == 0xFF)
		if(retry++ > 8) break;                  // card has timed out
	return resp;                                    // Note! MMC enable is ON
}

// Initialize mmc-card, returns zero if successful.
unsigned char mmc_init(void) 
{
	unsigned char retry, i, resp=0;

	retry = 0;
	do                                                 // max 10 retries
	{
		// send 74+ (=80) dummy bits with CS high before accessing        
		for(i=0;i<100;i++)         
			spi(0xFF);
		// resetting card, go to SPI mode
		resp = mmc_command(IDLE_STATE, 0);             // command 0
		retry++;
		if(retry>10)
		{
			return -1;
		}
			
	}
	while(resp != 0x01);

	retry = 0;
	
	do                                                 // max 100 retries
	{ 
		 // send init cmd (1)
		resp = mmc_command(INIT_COMMAND, 0); 
		delay_ms(5);
		retry++;
		if(retry>100) 
		{
			return -1;                
		}
	}
	while(resp);  

	// turn off CRC checking (not actually needed CRC-off is default)
	resp = mmc_command(CRC_OFF, 0);
	// set block length to 512 bytes (not actually needed 512 bytes is default)
	resp = mmc_command(SET_BLKLEN, BLOCKSIZE);    
	delay_ms(10);
	return 0;
}

// Read buffer, returns zero if successful.
unsigned char mmc_read(unsigned long sector, unsigned char* buffer) 
{
	unsigned char resp;
	unsigned int i;

	CS = 0;                                            		 		// set chip select
	resp = mmc_command(READ_BLK, sector<<9);		// send command:READ-BLOCK
	if(resp != 0x00)
		return resp;												// check response
	while(spi(0xFF) != STARTBYTE_RDWR);				// wait for block start
	for(i=0; i<BLOCKSIZE; i++)
	{      
		*buffer++ = spi(0xFF);									// read in data
	}
	spi(0xFF);														// read 16-bit CRC
	spi(0xFF);
	return 0;														// return OK
}

// write to MMC from buffer
unsigned char mmc_write(unsigned long sector, unsigned char* buffer)
{      
	unsigned char resp;
	unsigned int i;

	CS = 0;															// set chip select
	resp = mmc_command(WRITE_BLK, sector<<9);
	if(resp != 0x00)
		return resp;												// check for  response
	spi(0xFF);														// send dummy
	spi(STARTBYTE_RDWR);                                    // send data start token
	for(i=0; i<BLOCKSIZE; i++)
	{       
		spi(*buffer++);
	}
	spi(0xFF);														// write 16-bit CRC (dummy values)
	spi(0xFF);
	resp = spi(0xFF);
	if( (resp & DR_MASK) != DR_ACCEPT)    
		return resp;      											// read data response

	while(!spi(0xFF));											// wait until card not busy
	CS = 1;															// release chip select
	return 0;														// return ok
}
